# -*- perl -*-
#
# Copyright (C) 2013 Red Hat, Inc.
# Copyright (C) 2013 Zhe Peng <zpeng@redhat.com>
#
# This program is free software; You can redistribute it and/or modify
# it under the GNU General Public License as published by the Free
# Software Foundation; either version 2, or (at your option) any
# later version
#
# The file "LICENSE" distributed along with this file provides full
# details of the terms and conditions
#

=pod

=head1 NAME

domain/121-block-info.t

=head1 DESCRIPTION

The test case validates that all following APIs work well include
dom->block_resize
dom->get_block_info
dom->block_peek

=cut

use strict;
use warnings;

use Test::More;

use Sys::Virt::TCK;
use Test::Exception;
use File::stat;

my $tck = Sys::Virt::TCK->new();
my $conn = eval { $tck->setup(); };
BAIL_OUT "failed to setup test harness: $@" if $@;
END {
    $tck->cleanup if $tck;
}

if ( $conn->get_type() eq 'Xen' ) {
    $tck->cleanup if $tck;
    eval "use Test::More skip_all => \"xl missing function virDomainGetBlockInfo\";";
} else {
    eval "use Test::More tests => 32";
}

# test is_alive
my $live = $conn->is_alive();
ok($live > 0, "Connection is alive");

my $xml = $tck->generic_pool("dir")->as_xml;

diag "Defining transient storage pool";
my $pool;
ok_pool(sub { $pool = $conn->define_storage_pool($xml) }, "define transient storage pool");
lives_ok(sub { $pool->build(0) }, "built storage pool");
lives_ok(sub { $pool->create }, "started storage pool");

my $volallocxml1 = $tck->generic_volume("tck1", "raw", 1024*1024*50)->allocation(1024*1024*50)->as_xml;
my ($vol1, $path1, $st1);
ok_volume { $vol1 = $pool->create_volume($volallocxml1) } "create fully allocated raw volume";

my $volallocxml2 = $tck->generic_volume("tck2", "raw", 1024*1024*50)->allocation(1024*1024)->as_xml;
my ($vol2, $path2, $st2);
ok_volume { $vol2 = $pool->create_volume($volallocxml2) } "create not fully allocated raw volume";

my $volallocxml3 = $tck->generic_volume("tck3", "qcow2", 1024*1024*50)->allocation(1024*1024)->as_xml;
my ($vol3, $path3, $st3);
ok_volume { $vol3 = $pool->create_volume($volallocxml3) } "create qcow2 volume";

$path1 = xpath($vol1, "string(/volume/target/path)");
$st1 = stat($path1);
ok($st1, "path $path1 exists");
is($st1->size, 1024*1024*50, "size is 50M");

$path2 = xpath($vol2, "string(/volume/target/path)");
$st2 = stat($path2);
ok($st2, "path $path2 exists");

$path3 = xpath($vol3, "string(/volume/target/path)");
$st3 = stat($path3);
ok($st3, "path $path3 exists");

diag "Generic guest with previous created vol";

# Use hd* and tap devices under xen, otherwise vd* and qemu devices
my ($dev, $name) = ( $conn->get_type() eq 'Xen' ) ? ("hd", "tap") : ("vd", "qemu");
my $disktype = "raw";
my $dst1 = $dev."a";
my $dst2 = $dev."b";
my $dst3 = $dev."c";
my $guest = $tck->generic_domain(name => "tck");
$guest->rmdisk();

$guest->disk(format => { name => $name, type => $disktype }, type => "file", src => $path1, dst => $dst1);
$guest->disk(format => { name => $name, type => $disktype }, type => "file", src=> $path2, dst => $dst2);
$guest->disk(format => { name => $name, type => "qcow2" }, type => "file", src=> $path3, dst => $dst3);

$xml = $guest->as_xml;
my $dom;
ok_domain(sub { $dom = $conn->create_domain($xml) }, "Create domain");
$xml = $dom->get_xml_description();

# Fully allocated volume
is($dom->get_block_info($dst1,0)->{capacity}, 1024*1024*50, "Get disk capacity info");
ok($dom->get_block_info($dst1,0)->{allocation} >= 0, "Get disk allocation info");
ok($dom->get_block_info($dst1,0)->{physical} >= 1024*1024*50, "Get disk physical info");

# Not fully allocated (sparse) raw volume
my $sparse_alloc=(`libvirtd -V` !~ m/1\.[12]\.[0-9]$/) ? 0 : 1024*1024;
is($dom->get_block_info($dst2,0)->{capacity}, 1024*1024*50, "Get disk capacity info");
is($dom->get_block_info($dst2,0)->{allocation}, $sparse_alloc, "Get disk allocation info");
is($dom->get_block_info($dst2,0)->{physical}, 1024*1024, "Get disk physical info");

# qcow2 volume
is($dom->get_block_info($dst3,0)->{capacity}, 1024*1024*50, "Get disk capacity info");
ok($dom->get_block_info($dst3,0)->{allocation} >= 0, "Get disk allocation info");
ok($dom->get_block_info($dst3,0)->{physical} >= 512*392, "Get disk physical info");

diag "Test block_resize";
lives_ok(sub {$dom->block_resize($dst1, 512*50)}, "resize to 512*50 KB");
$st1 = stat($path1);
is($st1->size, 512*1024*50, "size is 25M");

is($dom->get_block_info($dst1,0)->{capacity}, 1024*512*50, "Get disk capacity info");
ok($dom->get_block_info($dst1,0)->{allocation} >= 0, "Get disk allocation info");
ok($dom->get_block_info($dst1,0)->{physical} >= 1024*512*50, "Get disk physical info");

lives_ok(sub {$dom->block_resize($dst1, 1024*50)}, "resize to 1024*50 KB");
$st1 = stat($path1);
is($st1->size, 1024*1024*50, "size is 50M");

diag "Test block_peek";
my $date = "test";
system("echo $date > $path1");
is($dom->block_peek($path1,0,4,0), $date, "Get date from raw image");

#qcow2 file start with hexadecimal:0x51 0x46 0x49 (ASCII: "QFI")
is($dom->block_peek($path3,0,3,0), "QFI", "Get date from qcow2 image");

lives_ok(sub { $vol1->delete(0) }, "deleted volume");

diag "Destroy domain";
$dom->destroy;

ok_error(sub { $conn->get_domain_by_name("tck") }, "NO_DOMAIN error raised from missing domain", 42);
